@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 
 import numpy as np
-from .optimizer import Optimizer, CandidateState, OptimizationResults 
+from .optimizer import Optimizer, CandidateState 
 from scipy.special import gamma
 
 """
@@ -29,13 +29,7 @@ class SSA(Optimizer):
         self.X = None
         self.X0 = None
         self.method = 'Vanilla'
-        self.swarm_size = None
         self.params = {}
-        
-        # Default params
-        self.Pdp = 0.1 # predator_presence_probability
-        self.Gc = 1.9 # gliding_constant
-        self.dg_lim = [0.5, 1.11] # gliding_distance_limits (for random dg)
 
     def _check_params(self):
 
@@ -43,26 +37,44 @@ class SSA(Optimizer):
         mandatory_params, optional_params = [], []
 
         if self.method == 'Vanilla':
-            mandatory_params = 'acorn_tree_attraction'.split()
-            optional_params = 'predator_presence_probability gliding_constant \
-                                gliding_distance_limits'.split()
+            mandatory_params = 'swarm_size acorn_tree_attraction'.split()
+            # the following params are better left at default
+            mandatory_params += 'predator_presence_probability gliding_constant \
+                                gliding_distance_limits'.split() 
+            if 'swarm_size' not in self.params:
+                self.params['swarm_size'] = self.dimensions
+                defined_params += 'swarm_size'.split()
+            if 'acorn_tree_attraction' not in self.params:
+                self.params['acorn_tree_attraction'] = 0.5
+                defined_params += 'acorn_tree_attraction'.split()
+            # the following params are better left at default
+            if 'predator_presence_probability' not in self.params:
+                self.params['predator_presence_probability'] = 0.1
+                defined_params += 'predator_presence_probability'.split()
+            if 'gliding_constant' not in self.params:
+                self.params['gliding_constant'] = 1.9
+                defined_params += 'gliding_constant'.split()
+            if 'gliding_distance_limits' not in self.params:
+                self.params['gliding_distance_limits'] = [0.5, 1.11]
+                defined_params += 'gliding_distance_limits'.split()
+            optional_params = ''.split()
+        else:
+            assert False, f'Unknown method! {self.method}'
 
         Optimizer._check_params(self, mandatory_params, optional_params, defined_params)
         
     def _init_method(self):
         
-        self._init_optimizer()
-        
         # Bounds for position and velocity
         self.lb = np.array(self.lb)
         self.ub = np.array(self.ub)
 
         # Generate a swarm of FS
-        self.cS = np.array([FlyingSquirrel(self) for c in range(self.swarm_size)], \
+        self.cS = np.array([FlyingSquirrel(self) for c in range(self.params['swarm_size'])], \
                             dtype=FlyingSquirrel)
         
         # Generate initial positions
-        for p in range(self.swarm_size):            
+        for p in range(self.params['swarm_size']):            
             # Random position
             self.cS[p].X = np.random.uniform(self.lb, self.ub)           
             # Using specified particles initial positions
@@ -75,30 +87,28 @@ class SSA(Optimizer):
         self.collective_evaluation(self.cS)
         
         # Find the overall best, i.e. FSht (hickory nut tree)
-        self.best = np.sort(self.cS)[0]
+        #self.best = np.sort(self.cS)[0] # already done in collective_evaluation
             
         # Update history
-        self.results = OptimizationResults(self)
-        self.results.cHistory = [self.best]
+        #self.results.cHistory = [self.best] # superseded by progress_log()
+        self._progress_log()
         
-    def run(self, seed=None):
-        
-        Optimizer.run(self, seed=seed)
+    def _run(self):
         self._check_params()
         self._init_method()
       
         # Load params
-        if 'acorn_tree_attraction' in self.params.keys():
+        if 'acorn_tree_attraction' in self.params:
             # part of FSnt moving to FSat
             # ATA=0 (all move to FSht) - emphasize local search
             # ATA=1 (all move to FSat's) - emphasize global search
-            self.ATA = self.params['acorn_tree_attraction']
-        if 'predator_presence_probability' in self.params.keys():
-            self.Pdp = self.params['predator_presence_probability']
-        if 'gliding_constant' in self.params.keys():
-            self.Gc = self.params['gliding_constant']
-        if 'gliding_distance_limits' in self.params.keys():
-            self.dg_lim = self.params['gliding_distance_limits']
+            ATA = self.params['acorn_tree_attraction']
+        if 'predator_presence_probability' in self.params:
+            Pdp = self.params['predator_presence_probability']
+        if 'gliding_constant' in self.params:
+            Gc = self.params['gliding_constant']
+        if 'gliding_distance_limits' in self.params:
+            dg_lim = self.params['gliding_distance_limits']
             
         def Levy():
             ra, rb = np.random.normal(0, 1), np.random.normal(0, 1)
@@ -107,7 +117,7 @@ class SSA(Optimizer):
                      gamma((1 + beta) / 2) * beta * 2**((beta - 1)/2)) **(1 / beta)
             return 0.01 * (ra * sigma) / (np.abs(rb)**(1 / beta))
 
-        for i in range(self.iterations):
+        for self.it in range(1, self.iterations + 1):
             
             # Categorizing FS's
             FSht = np.sort(self.cS)[0] # best FS (hickory nut trees)
@@ -119,39 +129,39 @@ class SSA(Optimizer):
             # move principally to FSat; 
             # with probability = (1-Pdp)*Pdp = 0.09 move to Fsht
             for fs in FSnt:
-                if np.random.rand() >= self.Pdp: # move towards FSat
-                    dg = np.random.uniform(self.dg_lim[0], self.dg_lim[1])
-                    fs.X = fs.X + dg * self.Gc * \
+                if np.random.rand() >= Pdp: # move towards FSat
+                    dg = np.random.uniform(dg_lim[0], dg_lim[1])
+                    fs.X = fs.X + dg * Gc * \
                             (np.random.choice(FSat).X - fs.X)
-                elif np.random.rand() >= self.Pdp: # move towards FSht
-                    dg = np.random.uniform(self.dg_lim[0], self.dg_lim[1])
-                    fs.X = fs.X + dg * self.Gc * (FSht.X - fs.X)
+                elif np.random.rand() >= Pdp: # move towards FSht
+                    dg = np.random.uniform(dg_lim[0], dg_lim[1])
+                    fs.X = fs.X + dg * Gc * (FSht.X - fs.X)
                 else: # not moving, i.e. respawning randomly
                     fs.X = np.random.uniform(self.lb, self.ub)
             """
             
             # Moving FSnt
-            Nnt2at = int(np.size(FSnt) * self.ATA) # attracted to acorn trees
+            Nnt2at = int(np.size(FSnt) * ATA) # attracted to acorn trees
             np.random.shuffle(FSnt)
             for fs in FSnt[:Nnt2at]:
-                if np.random.rand() >= self.Pdp: # move towards FSat
-                    dg = np.random.uniform(self.dg_lim[0], self.dg_lim[1])
-                    fs.X = fs.X + dg * self.Gc * \
+                if np.random.rand() >= Pdp: # move towards FSat
+                    dg = np.random.uniform(dg_lim[0], dg_lim[1])
+                    fs.X = fs.X + dg * Gc * \
                             (np.random.choice(FSat).X - fs.X)
                 else: # not moving, i.e. respawning randomly
                     fs.X = np.random.uniform(self.lb, self.ub)
             for fs in FSnt[Nnt2at:]:
-                if np.random.rand() >= self.Pdp: # move towards FSht
-                    dg = np.random.uniform(self.dg_lim[0], self.dg_lim[1])
-                    fs.X = fs.X + dg * self.Gc * (FSht.X - fs.X)
+                if np.random.rand() >= Pdp: # move towards FSht
+                    dg = np.random.uniform(dg_lim[0], dg_lim[1])
+                    fs.X = fs.X + dg * Gc * (FSht.X - fs.X)
                 else: # not moving, i.e. respawning randomly
                     fs.X = np.random.uniform(self.lb, self.ub)
             
             # Moving FSat
             for fs in FSat:
-                if np.random.rand() >= self.Pdp: # move towards FSht
-                    dg = np.random.uniform(self.dg_lim[0], self.dg_lim[1])
-                    fs.X = fs.X + dg * self.Gc * (FSht.X - fs.X)
+                if np.random.rand() >= Pdp: # move towards FSht
+                    dg = np.random.uniform(dg_lim[0], dg_lim[1])
+                    fs.X = fs.X + dg * Gc * (FSht.X - fs.X)
                 else: # not moving, i.e. respawning randomly
                     fs.X = np.random.uniform(self.lb, self.ub)
             
@@ -161,7 +171,7 @@ class SSA(Optimizer):
                 Sc[i] = np.sqrt(np.sum((fs.X - FSht.X)**2))
                 
             # Minimum value of seasonal constant
-            Scmin = 1e-6 / (365**(i / (self.iterations / 2.5))) # this is some black magic shit
+            Scmin = 1e-6 / (365**(self.it / (self.iterations / 2.5))) # this is some black magic shit
             
             # Random-Levy relocation at the end of winter season
             if (Sc < Scmin).all():
@@ -171,10 +181,6 @@ class SSA(Optimizer):
             for cP in self.cS:
                 # Correct position to the bounds
                 cP.X = np.clip(cP.X, self.lb, self.ub)       
-
-            # # Evaluate swarm, one by one
-            # for cP in self.cS:
-            #     cP.evaluate()
                 
             # Evaluate swarm
             self.collective_evaluation(self.cS)
@@ -183,13 +189,11 @@ class SSA(Optimizer):
             self.best = np.min(self.cS)
              
             # Update history
-            self.results.cHistory.append(self.best.copy())
-
-            if self.verbose:
-                print(i, self.best.f)
+            #self.results.cHistory.append(self.best.copy()) # superseded by progress_log()
+            self._progress_log()
                 
-            # Stop if fitness threshold achieved
-            if self.best.f < self.target_fitness:
+            # Check stopping conditions
+            if self._stopping_criteria():
                 break
-
+            
         return self.best
\ No newline at end of file
